class GlobalSensitivity(object):
    def __init__(self, value):
        super().__init__()
        self.value = value

    @property
    def value(self):
        return self.__value

    @value.setter
    def value(self, value):
        self.__value = value



import unittest
from diffpriv_laplace.global_sensitivity.counting import CountingGlobalSensitivity


class TestCountingGlobalSensitivity(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_value(self):
        gs = CountingGlobalSensitivity()
        self.assertEqual(gs.value, 1.0)



from diffpriv_laplace.global_sensitivity.max import MaxGlobalSensitivity
from diffpriv_laplace.anonymizer.base import DiffPrivAnonymizer


class DiffPrivMaxAnonymizer(DiffPrivAnonymizer):
    def __init__(self, epsilon):
        gs = MaxGlobalSensitivity()
        super().__init__(gs, epsilon)



import unittest
from diffpriv_laplace.global_sensitivity.base import GlobalSensitivity


class TestGlobalSensitivity(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_getter(self):
        value = 5
        gs = GlobalSensitivity(value)
        self.assertEqual(gs.value, value)

    def test_setter(self):
        value = 5
        gs = GlobalSensitivity(value)
        new_value = 6
        gs.value = new_value
        self.assertEqual(gs.value, new_value)



import unittest
from diffpriv_laplace.global_sensitivity.mean import MeanGlobalSensitivity


class TestMeanGlobalSensitivity(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_value(self):
        lower = 10.0
        upper = 99.0
        n = 100.0
        value = (upper - lower) / n
        gs = MeanGlobalSensitivity(lower, upper, n)
        self.assertEqual(gs.value, value)

    def test_length_getter(self):
        lower = 10.0
        upper = 99.0
        n = 100.0
        gs = MeanGlobalSensitivity(lower, upper, n)
        self.assertEqual(gs.n, n)

    def test_lower_getter(self):
        lower = 10.0
        upper = 99.0
        n = 100.0
        gs = MeanGlobalSensitivity(lower, upper, n)
        self.assertEqual(gs.lower, lower)

    def test_upper_getter(self):
        lower = 10.0
        upper = 99.0
        n = 100.0
        gs = MeanGlobalSensitivity(lower, upper, n)
        self.assertEqual(gs.upper, upper)



import numpy as np


class DiffPrivAnonymizer(object):
    @classmethod
    def calculate_scale(cls, gs, epsilon):
        scale = gs.value / epsilon
        return scale

    def __init__(self, gs, epsilon):
        super().__init__()
        self.__gs = gs
        self.__epsilon = float(epsilon)
        self.__scale = self.calculate_scale(self.__gs, self.__epsilon)

    def apply(self, values, size=None):
        samples = np.random.laplace(loc=values, scale=self.__scale, size=size)
        return samples

    @property
    def scale(self):
        return self.__scale

    @property
    def global_sensitivity(self):
        return self.__gs

    @property
    def epsilon(self):
        return self.__epsilon



import unittest
import numpy as np
from diffpriv_laplace.anonymizer.proportion import DiffPrivProportionAnonymizer


class TestDiffPrivProportionAnonymizer(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def set_seed(self):
        np.random.seed(31337)

    def test_length_getter(self):
        n = 10.0
        epsilon = 1.0
        anonymizer = DiffPrivProportionAnonymizer(epsilon, n)
        self.assertEqual(anonymizer.n, n)

    def test_global_sensitivity_getter(self):
        n = 10.0
        epsilon = 1.0
        gs_value = 1.0 / n
        anonymizer = DiffPrivProportionAnonymizer(epsilon, n)
        np.testing.assert_almost_equal(anonymizer.global_sensitivity.value, gs_value)

    def test_epsilon_getter(self):
        n = 10.0
        epsilon = 1.0
        anonymizer = DiffPrivProportionAnonymizer(epsilon, n)
        self.assertEqual(anonymizer.epsilon, epsilon)

    def test_scale_getter(self):
        n = 10.0
        gs_value = 1.0 / n
        epsilon = 1.0
        scale = gs_value / epsilon
        anonymizer = DiffPrivProportionAnonymizer(epsilon, n)
        self.assertEqual(anonymizer.scale, scale)

    def test_apply_single(self):
        expected_value = 87.05864551385037
        n = 10.0
        epsilon = 1.0
        anonymizer = DiffPrivProportionAnonymizer(epsilon, n)
        self.set_seed()
        value = anonymizer.apply(87.0)
        np.testing.assert_almost_equal(value, expected_value)

    def test_apply_single_many(self):
        expected_values = np.array([87.0586455, 87.2701297, 86.9451988])
        n = 10.0
        epsilon = 1.0
        anonymizer = DiffPrivProportionAnonymizer(epsilon, n)
        self.set_seed()
        values = anonymizer.apply(87.0, size=3)
        np.testing.assert_almost_equal(values, expected_values)

    def test_apply_multiple(self):
        expected_values = np.array([87.0586455, 435.2701297])
        n = 10.0
        epsilon = 1.0
        anonymizer = DiffPrivProportionAnonymizer(epsilon, n)
        self.set_seed()
        values = anonymizer.apply([87.0, 435.0])
        np.testing.assert_almost_equal(values, expected_values)



import unittest
from diffpriv_laplace.global_sensitivity.proportion import ProportionGlobalSensitivity


class TestProportionGlobalSensitivity(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_value(self):
        n = 100.0
        value = 1.0 / n
        gs = ProportionGlobalSensitivity(n)
        self.assertEqual(gs.value, value)

    def test_length_getter(self):
        n = 100.0
        gs = ProportionGlobalSensitivity(n)
        self.assertEqual(gs.n, n)



import unittest
import numpy as np
from diffpriv_laplace.global_sensitivity.sum import SumGlobalSensitivity


class TestSumGlobalSensitivity(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_value(self):
        lower = 10.0
        upper = 99.0
        value = np.max([np.abs(lower), np.abs(upper)])
        gs = SumGlobalSensitivity(lower, upper)
        self.assertEqual(gs.value, value)

    def test_lower_getter(self):
        lower = 10.0
        upper = 99.0
        gs = SumGlobalSensitivity(lower, upper)
        self.assertEqual(gs.lower, lower)

    def test_upper_getter(self):
        lower = 10.0
        upper = 99.0
        gs = SumGlobalSensitivity(lower, upper)
        self.assertEqual(gs.upper, upper)



from diffpriv_laplace.global_sensitivity.median import MedianGlobalSensitivity
from diffpriv_laplace.anonymizer.base import DiffPrivAnonymizer


class DiffPrivMedianAnonymizer(DiffPrivAnonymizer):
    def __init__(self, epsilon):
        gs = MedianGlobalSensitivity()
        super().__init__(gs, epsilon)



from diffpriv_laplace.laplace_mechanism import DiffPrivLaplaceMechanism
from diffpriv_laplace.statistics import DiffPrivStatistics, DiffPrivStatisticKind
from diffpriv_laplace.version import __version__

__all__ = [
    "DiffPrivLaplaceMechanism",
    "DiffPrivStatistics",
    "DiffPrivStatisticKind",
    "__version__",
]



import numpy as np
from diffpriv_laplace.global_sensitivity.base import GlobalSensitivity


class VarianceGlobalSensitivity(GlobalSensitivity):
    @classmethod
    def calculate_value(cls, lower, upper, n):
        value = np.square(upper - lower) / n
        return value

    def __init__(self, lower, upper, n):
        self.__lower = float(lower)
        self.__upper = float(upper)
        self.__n = float(n)
        super().__init__(self.calculate_value(self.__lower, self.__upper, self.__n))

    @property
    def lower(self):
        return self.__lower

    @property
    def upper(self):
        return self.__upper

    @property
    def n(self):
        return self.__n



from diffpriv_laplace.global_sensitivity.min import MinGlobalSensitivity
from diffpriv_laplace.anonymizer.base import DiffPrivAnonymizer


class DiffPrivMinAnonymizer(DiffPrivAnonymizer):
    def __init__(self, epsilon):
        gs = MinGlobalSensitivity()
        super().__init__(gs, epsilon)



from diffpriv_laplace.global_sensitivity.base import GlobalSensitivity


class ProportionGlobalSensitivity(GlobalSensitivity):
    @classmethod
    def calculate_value(cls, n):
        value = 1.0 / n
        return value

    def __init__(self, n):
        self.__n = float(n)
        super().__init__(self.calculate_value(self.__n))

    @property
    def n(self):
        return self.__n



from diffpriv_laplace.global_sensitivity.proportion import ProportionGlobalSensitivity
from diffpriv_laplace.anonymizer.base import DiffPrivAnonymizer


class DiffPrivProportionAnonymizer(DiffPrivAnonymizer):
    def __init__(self, epsilon, n):
        gs = ProportionGlobalSensitivity(n)
        super().__init__(gs, epsilon)

    @property
    def n(self):
        return self.global_sensitivity.n



import numpy as np
from enum import Flag, auto
from diffpriv_laplace import DiffPrivLaplaceMechanism


class DiffPrivStatisticKind(Flag):
    """
    The kind flag representing each statistic operation.
    """

    count = auto()
    min = auto()
    max = auto()
    median = auto()
    proportion = auto()
    sum = auto()
    mean = auto()
    variance = auto()
    all = count | min | max | median | proportion | sum | mean | variance

    @property
    def size(self):
        """
        Calculates the amount of statistic kind in the flag.

        Returns
        -------
        int
            The total amount of statistic kind in the flag.

        """
        value = 0
        if self == DiffPrivStatisticKind.all:
            value = len(DiffPrivStatisticKind) - 1
        else:
            for kind in DiffPrivStatisticKind:
                if kind != self.all and bool(self & kind):
                    value = value + 1

        return value


class DiffPrivStatistics(object):
    """
    Differential privacy Laplace mechanism statistics.
    """

    @classmethod
    def count(cls, data, epsilon, condition=None, axis=None):
        """
        Performs the count operation and anonymizes the value(s) using the provided
        privacy budget.

        The operation counts non-zero or non-False values. When a condition
        function is provided, it is invoked so that the data can be transformed
        before counting.

        Parameters
        ----------
        data : list|ndarray
            The data to retrieve the count(s) from.
        epsilon : float
            The privacy budget.
        [condition] : function
            A condition function that receives the data as a parameter and returns
            a list or ndarray of the same size in which for each element is either
            a zero or False when it should not be counted and non-zero or True when
            it should.
        [axis] : int|tuple
            Axis or tuple of axes along which to count non-zeros or non-False value(s).

        Returns
        -------
        float|ndarray
            The anonymized count(s).

        """
        if condition:
            data = condition(data)

        value = np.count_nonzero(data, axis=axis)
        anonymized = DiffPrivLaplaceMechanism.anonymize_count_with_budget(
            value, epsilon
        )
        return anonymized

    @classmethod
    def min(cls, data, epsilon, axis=None):
        """
        Performs the min operation and anonymizes the value(s) using the provided
        privacy budget.

        Parameters
        ----------
        data : list|ndarray
            The data to retrieve the min value(s) from.
        epsilon : float
            The privacy budget.
        [axis] : int|tuple
            Axis or tuple of axes along which to obtain the min value(s).

        Returns
        -------
        float|ndarray
            The anonymized min value(s).

        """
        value = np.min(data, axis=axis)
        anonymized = DiffPrivLaplaceMechanism.anonymize_min_with_budget(value, epsilon)
        return anonymized

    @classmethod
    def max(cls, data, epsilon, axis=None):
        """
        Performs the max operation and anonymizes the value(s) using the provided
        privacy budget.

        Parameters
        ----------
        data : list|ndarray
            The data to retrieve the max value(s) from.
        epsilon : float
            The privacy budget.
        [axis] : int|tuple
            Axis or tuple of axes along which to obtain the max value(s).

        Returns
        -------
        float|ndarray
            The anonymized max value(s).

        """
        value = np.max(data, axis=axis)
        anonymized = DiffPrivLaplaceMechanism.anonymize_max_with_budget(value, epsilon)
        return anonymized

    @classmethod
    def median(cls, data, epsilon, axis=None):
        """
        Performs the median operation and anonymizes the value(s) using the provided
        privacy budget.

        Parameters
        ----------
        data : list|ndarray
            The data to retrieve the median value(s) from.
        epsilon : float
            The privacy budget.
        [axis] : int|tuple
            Axis or tuple of axes along which to obtain the median value(s).

        Returns
        -------
        float|ndarray
            The anonymized median value(s).

        """
        value = np.median(data, axis=axis)
        anonymized = DiffPrivLaplaceMechanism.anonymize_median_with_budget(
            value, epsilon
        )
        return anonymized

    @classmethod
    def proportion(cls, data, epsilon, condition=None, axis=None):
        """
        Performs the proportion operation and anonymizes the value(s) using the
        provided privacy budget.

        The operation counts non-zero or non-False values. When a condition
        function is provided, it is invoked so that the data can be transformed
        before counting. Afterwords, the proportion is calculated using the total
        amount of values.

        Parameters
        ----------
        data : list|ndarray
            The data to retrieve the proportion(s) from.
        epsilon : float
            The privacy budget.
        [condition] : function
            A condition function that receives the data as a parameter and returns
            a list or ndarray of the same size in which for each element is either
            a zero or False when it should not be counted and non-zero or True when
            it should.
        [axis] : int|tuple
            Axis or tuple of axes along which to count non-zeros or non-False value(s).

        Returns
        -------
        float|ndarray
            The anonymized count(s).

        """
        n = np.size(data, axis=axis)
        if condition:
            data = condition(data)

        value = np.count_nonzero(data, axis=axis)
        value = np.divide(value, n)
        anonymized = DiffPrivLaplaceMechanism.anonymize_proportion_with_budget(
            value, n, epsilon
        )
        return anonymized

    @classmethod
    def sum(cls, data, epsilon, axis=None):
        """
        Performs the sum operation and anonymizes the value(s) using the provided
        privacy budget.

        Parameters
        ----------
        data : list|ndarray
            The data to retrieve the sum value(s) from.
        epsilon : float
            The privacy budget.
        [axis] : int|tuple
            Axis or tuple of axes along which to obtain the sum value(s).

        Returns
        -------
        float|ndarray
            The anonymized sum value(s).

        """
        lower = np.min(data, axis=axis)
        upper = np.max(data, axis=axis)
        value = np.sum(data, axis=axis)
        if np.isscalar(lower):
            anonymized = DiffPrivLaplaceMechanism.anonymize_sum_with_budget(
                value, lower, upper, epsilon
            )
        else:
            size = lower.size
            anonymized = np.zeros(size)
            for index in range(0, size):
                anonymized[index] = DiffPrivLaplaceMechanism.anonymize_sum_with_budget(
                    value[index], lower[index], upper[index], epsilon
                )

        return anonymized

    @classmethod
    def mean(cls, data, epsilon, axis=None):
        """
        Performs the mean operation and anonymizes the value(s) using the provided
        privacy budget.

        Parameters
        ----------
        data : list|ndarray
            The data to retrieve the mean value(s) from.
        epsilon : float
            The privacy budget.
        [axis] : int|tuple
            Axis or tuple of axes along which to obtain the mean value(s).

        Returns
        -------
        float|ndarray
            The anonymized mean value(s).

        """
        n = np.size(data, axis=axis)
        lower = np.min(data, axis=axis)
        upper = np.max(data, axis=axis)
        value = np.mean(data, axis=axis)
        if np.isscalar(lower):
            anonymized = DiffPrivLaplaceMechanism.anonymize_mean_with_budget(
                value, lower, upper, n, epsilon
            )
        else:
            size = lower.size
            anonymized = np.zeros(size)
            for index in range(0, size):
                mean_value = DiffPrivLaplaceMechanism.anonymize_mean_with_budget(
                    value[index], lower[index], upper[index], n, epsilon
                )
                anonymized[index] = mean_value

        return anonymized

    @classmethod
    def variance(cls, data, epsilon, axis=None):
        """
        Performs the variance operation and anonymizes the value(s) using the provided
        privacy budget.

        Parameters
        ----------
        data : list|ndarray
            The data to retrieve the variance value(s) from.
        epsilon : float
            The privacy budget.
        [axis] : int|tuple
            Axis or tuple of axes along which to obtain the variance value(s).

        Returns
        -------
        float|ndarray
            The anonymized variance value(s).

        """
        n = np.size(data, axis=axis)
        lower = np.min(data, axis=axis)
        upper = np.max(data, axis=axis)
        value = np.var(data, axis=axis)
        if np.isscalar(lower):
            anonymized = DiffPrivLaplaceMechanism.anonymize_variance_with_budget(
                value, lower, upper, n, epsilon
            )
        else:
            size = lower.size
            anonymized = np.zeros(size)
            for index in range(0, size):
                var_value = DiffPrivLaplaceMechanism.anonymize_variance_with_budget(
                    value[index], lower[index], upper[index], n, epsilon
                )
                anonymized[index] = var_value

        return anonymized



from enum import Enum
from diffpriv_laplace.anonymizer.count import DiffPrivCountAnonymizer
from diffpriv_laplace.anonymizer.counting import DiffPrivCountingAnonymizer
from diffpriv_laplace.anonymizer.min import DiffPrivMinAnonymizer
from diffpriv_laplace.anonymizer.max import DiffPrivMaxAnonymizer
from diffpriv_laplace.anonymizer.median import DiffPrivMedianAnonymizer
from diffpriv_laplace.anonymizer.proportion import DiffPrivProportionAnonymizer
from diffpriv_laplace.anonymizer.sum import DiffPrivSumAnonymizer
from diffpriv_laplace.anonymizer.mean import DiffPrivMeanAnonymizer
from diffpriv_laplace.anonymizer.variance import DiffPrivVarianceAnonymizer


class DiffPrivAnonymizerType(Enum):
    count = "count"
    counting = "counting"
    min = "min"
    max = "max"
    median = "median"
    proportion = "proportion"
    sum = "sum"
    mean = "mean"
    variance = "variance"


class DiffPrivLaplaceMechanism(object):
    """
    Differential privacy Laplace mechanism.
    """

    @classmethod
    def create_count_anonymizer(cls, epsilon):
        """
        Creates a count anonymizer instance.

        Parameters
        ----------
        epsilon : float
            The privacy budget.

        Returns
        -------
        `DiffPrivCountAnonymizer`
            A count anonymizer with the provided privacy budget.

        """
        anonymizer = DiffPrivCountAnonymizer(epsilon)
        return anonymizer

    @classmethod
    def create_counting_anonymizer(cls, epsilon):
        """
        Creates a counting anonymizer instance.

        Parameters
        ----------
        epsilon : float
            The privacy budget.

        Returns
        -------
        `DiffPrivCountingAnonymizer`
            A counting anonymizer with the provided privacy budget.

        """
        anonymizer = DiffPrivCountingAnonymizer(epsilon)
        return anonymizer

    @classmethod
    def create_min_anonymizer(cls, epsilon):
        """
        Creates a min anonymizer instance.

        Parameters
        ----------
        epsilon : float
            The privacy budget.

        Returns
        -------
        `DiffPrivMinAnonymizer`
            A min anonymizer with the provided privacy budget.

        """
        anonymizer = DiffPrivMinAnonymizer(epsilon)
        return anonymizer

    @classmethod
    def create_max_anonymizer(cls, epsilon):
        """
        Creates a max anonymizer instance.

        Parameters
        ----------
        epsilon : float
            The privacy budget.

        Returns
        -------
        `DiffPrivMaxAnonymizer`
            A max anonymizer with the provided privacy budget.

        """
        anonymizer = DiffPrivMaxAnonymizer(epsilon)
        return anonymizer

    @classmethod
    def create_median_anonymizer(cls, epsilon):
        """
        Creates a median anonymizer instance.

        Parameters
        ----------
        epsilon : float
            The privacy budget.

        Returns
        -------
        `DiffPrivMedianAnonymizer`
            A median anonymizer with the provided privacy budget.

        """
        anonymizer = DiffPrivMedianAnonymizer(epsilon)
        return anonymizer

    @classmethod
    def create_proportion_anonymizer(cls, epsilon, n):
        """
        Creates a proportion anonymizer instance.

        Parameters
        ----------
        epsilon : float
            The privacy budget.
        n : float
            The total number of observations.

        Returns
        -------
        `DiffPrivProportionAnonymizer`
            A proportion anonymizer with the provided privacy budget.

        """
        anonymizer = DiffPrivProportionAnonymizer(epsilon, n)
        return anonymizer

    @classmethod
    def create_sum_anonymizer(cls, epsilon, lower, upper):
        """
        Creates a sum anonymizer instance.

        Parameters
        ----------
        epsilon : float
            The privacy budget.
        lower : float
            The lower bound of the data.
        upper : float
            The upper bound of the data.

        Returns
        -------
        `DiffPrivSumAnonymizer`
            A sum anonymizer with the provided privacy budget.

        """
        anonymizer = DiffPrivSumAnonymizer(epsilon, lower, upper)
        return anonymizer

    @classmethod
    def create_mean_anonymizer(cls, epsilon, lower, upper, n):
        """
        Creates a mean anonymizer instance.

        Parameters
        ----------
        epsilon : float
            The privacy budget.
        lower : float
            The lower bound of the data.
        upper : float
            The upper bound of the data.
        n : float
            The total number of observations.

        Returns
        -------
        `DiffPrivMeanAnonymizer`
            A mean anonymizer with the provided privacy budget.

        """
        anonymizer = DiffPrivMeanAnonymizer(epsilon, lower, upper, n)
        return anonymizer

    @classmethod
    def create_variance_anonymizer(cls, epsilon, lower, upper, n):
        """
        Creates a variance anonymizer instance.

        Parameters
        ----------
        epsilon : float
            The privacy budget.
        lower : float
            The lower bound of the data.
        upper : float
            The upper bound of the data.
        n : float
            The total number of observations.

        Returns
        -------
        `DiffPrivVarianceAnonymizer`
            A variance anonymizer with the provided privacy budget.

        """
        anonymizer = DiffPrivVarianceAnonymizer(epsilon, lower, upper, n)
        return anonymizer

    @classmethod
    def anonymize_count_with_budget(cls, value, epsilon, size=None):
        """
        Anonymizes one or many count value(s) for a given privacy budget.

        Parameters
        ----------
        value : float|list|ndarray
            The count value(s).
        epsilon : float
            The privacy budget.
        [size] : int|tuple
            Output shape.

        Returns
        -------
        float|list|ndarray
            The anonymized count(s).

        """
        anonymizer = cls.create_counting_anonymizer(epsilon)
        anonymized = anonymizer.apply(value, size=size)
        return anonymized

    @classmethod
    def anonymize_min_with_budget(cls, value, epsilon, size=None):
        """
        Anonymizes one or many min value(s) for a given privacy budget.

        Parameters
        ----------
        value : float|list|ndarray
            The min value(s).
        epsilon : float
            The privacy budget.
        [size] : int|tuple
            Output shape.

        Returns
        -------
        float|list|ndarray
            The anonymized min value(s).

        """
        anonymizer = cls.create_min_anonymizer(epsilon)
        anonymized = anonymizer.apply(value, size=size)
        return anonymized

    @classmethod
    def anonymize_max_with_budget(cls, value, epsilon, size=None):
        """
        Anonymizes one or many max value(s) for a given privacy budget.

        Parameters
        ----------
        value : float|list|ndarray
            The max value(s).
        epsilon : float
            The privacy budget.
        [size] : int|tuple
            Output shape.

        Returns
        -------
        float|list|ndarray
            The anonymized max value(s).

        """
        anonymizer = cls.create_max_anonymizer(epsilon)
        anonymized = anonymizer.apply(value, size=size)
        return anonymized

    @classmethod
    def anonymize_median_with_budget(cls, value, epsilon, size=None):
        """
        Anonymizes one or many median value(s) for a given privacy budget.

        Parameters
        ----------
        value : float|list|ndarray
            The median value(s).
        epsilon : float
            The privacy budget.
        [size] : int|tuple
            Output shape.

        Returns
        -------
        float|list|ndarray
            The anonymized median value(s).

        """
        anonymizer = cls.create_median_anonymizer(epsilon)
        anonymized = anonymizer.apply(value, size=size)
        return anonymized

    @classmethod
    def anonymize_proportion_with_budget(cls, value, n, epsilon, size=None):
        """
        Anonymizes one or many proportion value(s) for a given privacy budget.

        Parameters
        ----------
        value : float|list|ndarray
            The proportion value(s).
        n : float
            The total number of observations.
        epsilon : float
            The privacy budget.
        [size] : int|tuple
            Output shape.

        Returns
        -------
        float|list|ndarray
            The anonymized proportion(s).

        """
        anonymizer = cls.create_proportion_anonymizer(epsilon, n)
        anonymized = anonymizer.apply(value, size=size)
        return anonymized

    @classmethod
    def anonymize_sum_with_budget(cls, value, lower, upper, epsilon, size=None):
        """
        Anonymizes one or many sum value(s) for a given privacy budget.

        Parameters
        ----------
        value : float|list|ndarray
            The sum value(s).
        lower : float
            The lower bound of the data.
        upper : float
            The upper bound of the data.
        epsilon : float
            The privacy budget.
        [size] : int|tuple
            Output shape.

        Returns
        -------
        float|list|ndarray
            The anonymized sum value(s).

        """
        anonymizer = cls.create_sum_anonymizer(epsilon, lower, upper)
        anonymized = anonymizer.apply(value, size=size)
        return anonymized

    @classmethod
    def anonymize_mean_with_budget(cls, value, lower, upper, n, epsilon, size=None):
        """
        Anonymizes one or many mean value(s) for a given privacy budget.

        Parameters
        ----------
        value : float|list|ndarray
            The mean value(s).
        lower : float
            The lower bound of the data.
        upper : float
            The upper bound of the data.
        n : float
            The total number of observations.
        epsilon : float
            The privacy budget.
        [size] : int|tuple
            Output shape.

        Returns
        -------
        float|list|ndarray
            The anonymized mean(s).

        """
        anonymizer = cls.create_mean_anonymizer(epsilon, lower, upper, n)
        anonymized = anonymizer.apply(value, size=size)
        return anonymized

    @classmethod
    def anonymize_variance_with_budget(cls, value, lower, upper, n, epsilon, size=None):
        """
        Anonymizes one or many variance value(s) for a given privacy budget.

        Parameters
        ----------
        value : float|list|ndarray
            The variance value(s).
        lower : float
            The lower bound of the data.
        upper : float
            The upper bound of the data.
        n : float
            The total number of observations.
        epsilon : float
            The privacy budget.
        [size] : int|tuple
            Output shape.

        Returns
        -------
        float|list|ndarray
            The anonymized variance(s).

        """
        anonymizer = cls.create_variance_anonymizer(epsilon, lower, upper, n)
        anonymized = anonymizer.apply(value, size=size)
        return anonymized

    def __init__(self, epsilon):
        """
        Initialize the Laplace mechanism with a privacy budget.

        Parameters
        ----------
        epsilon : float
            The privacy budget value to use.

        """
        self.__epsilon = float(epsilon)
        super().__init__()

    @property
    def epsilon(self):
        """
        The privacy budget value.

        Returns
        -------
        float
            The privacy budget value.

        """
        return self.__epsilon

    def anonymize_count(self, value, size=None):
        """
        Anonymizes one or many count value(s).

        Parameters
        ----------
        value : float|list|ndarray
            The count value(s).
        [size] : int|tuple
            Output shape.

        Returns
        -------
        float|list|ndarray
            The anonymized count(s).

        """
        anonymized = DiffPrivLaplaceMechanism.anonymize_count_with_budget(
            value, self.__epsilon, size=size
        )
        return anonymized

    def anonymize_min(self, value, size=None):
        """
        Anonymizes one or many min value(s).

        Parameters
        ----------
        value : float|list|ndarray
            The min value(s).
        [size] : int|tuple
            Output shape.

        Returns
        -------
        float|list|ndarray
            The anonymized min value(s).

        """
        anonymized = DiffPrivLaplaceMechanism.anonymize_min_with_budget(
            value, self.__epsilon, size=size
        )
        return anonymized

    def anonymize_max(self, value, size=None):
        """
        Anonymizes one or many max value(s).

        Parameters
        ----------
        value : float|list|ndarray
            The max value(s).
        [size] : int|tuple
            Output shape.

        Returns
        -------
        float|list|ndarray
            The anonymized max value(s).

        """
        anonymized = DiffPrivLaplaceMechanism.anonymize_max_with_budget(
            value, self.__epsilon, size=size
        )
        return anonymized

    def anonymize_median(self, value, size=None):
        """
        Anonymizes one or many ,median value(s).

        Parameters
        ----------
        value : float|list|ndarray
            The median value(s).
        [size] : int|tuple
            Output shape.

        Returns
        -------
        float|list|ndarray
            The anonymized median value(s).

        """
        anonymized = DiffPrivLaplaceMechanism.anonymize_median_with_budget(
            value, self.__epsilon, size=size
        )
        return anonymized

    def anonymize_proportion(self, value, n, size=None):
        """
        Anonymizes one or many proportion value(s).

        Parameters
        ----------
        value : float|list|ndarray
            The proportion value(s).
        n : float
            The total number of observations.
        [size] : int|tuple
            Output shape.

        Returns
        -------
        float|list|ndarray
            The anonymized proportion(s).

        """
        anonymized = DiffPrivLaplaceMechanism.anonymize_proportion_with_budget(
            value, n, self.__epsilon, size=size
        )
        return anonymized

    def anonymize_sum(self, value, lower, upper, size=None):
        """
        Anonymizes one or many sum value(s).

        Parameters
        ----------
        value : float|list|ndarray
            The sum value(s).
        lower : float
            The lower bound of the data.
        upper : float
            The upper bound of the data.
        [size] : int|tuple
            Output shape.

        Returns
        -------
        float|list|ndarray
            The anonymized sum value(s).

        """
        anonymized = DiffPrivLaplaceMechanism.anonymize_sum_with_budget(
            value, lower, upper, self.__epsilon, size=size
        )
        return anonymized

    def anonymize_mean(self, value, lower, upper, n, size=None):
        """
        Anonymizes one or many mean value(s).

        Parameters
        ----------
        value : float|list|ndarray
            The mean value(s).
        lower : float
            The lower bound of the data.
        upper : float
            The upper bound of the data.
        n : float
            The total number of observations.
        [size] : int|tuple
            Output shape.

        Returns
        -------
        float|list|ndarray
            The anonymized mean(s).

        """
        anonymized = DiffPrivLaplaceMechanism.anonymize_mean_with_budget(
            value, lower, upper, n, self.__epsilon, size=size
        )
        return anonymized

    def anonymize_variance(self, value, lower, upper, n, size=None):
        """
        Anonymizes one or many variance value(s).

        Parameters
        ----------
        value : float|list|ndarray
            The variance value(s).
        lower : float
            The lower bound of the data.
        upper : float
            The upper bound of the data.
        n : float
            The total number of observations.
        [size] : int|tuple
            Output shape.

        Returns
        -------
        float|list|ndarray
            The anonymized variance(s).

        """
        anonymized = DiffPrivLaplaceMechanism.anonymize_variance_with_budget(
            value, lower, upper, n, self.__epsilon, size=size
        )
        return anonymized



import unittest
import numpy as np
from diffpriv_laplace.anonymizer.count import DiffPrivCountAnonymizer
from diffpriv_laplace.anonymizer.counting import DiffPrivCountingAnonymizer
from diffpriv_laplace.anonymizer.min import DiffPrivMinAnonymizer
from diffpriv_laplace.anonymizer.max import DiffPrivMaxAnonymizer
from diffpriv_laplace.anonymizer.median import DiffPrivMedianAnonymizer
from diffpriv_laplace.anonymizer.proportion import DiffPrivProportionAnonymizer
from diffpriv_laplace.anonymizer.sum import DiffPrivSumAnonymizer
from diffpriv_laplace.anonymizer.mean import DiffPrivMeanAnonymizer
from diffpriv_laplace.anonymizer.variance import DiffPrivVarianceAnonymizer
from diffpriv_laplace import DiffPrivLaplaceMechanism


class TestDiffPrivLaplaceMechanism(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def set_seed(self):
        np.random.seed(31337)

    def test_create_count_anonymizer(self):
        epsilon = 0.1
        anonymizer = DiffPrivLaplaceMechanism.create_count_anonymizer(epsilon)
        self.assertIsInstance(anonymizer, DiffPrivCountAnonymizer)
        self.assertEqual(anonymizer.epsilon, epsilon)

    def test_create_counting_anonymizer(self):
        epsilon = 0.1
        anonymizer = DiffPrivLaplaceMechanism.create_counting_anonymizer(epsilon)
        self.assertIsInstance(anonymizer, DiffPrivCountingAnonymizer)
        self.assertEqual(anonymizer.epsilon, epsilon)

    def test_create_min_anonymizer(self):
        epsilon = 0.1
        anonymizer = DiffPrivLaplaceMechanism.create_min_anonymizer(epsilon)
        self.assertIsInstance(anonymizer, DiffPrivMinAnonymizer)
        self.assertEqual(anonymizer.epsilon, epsilon)

    def test_create_max_anonymizer(self):
        epsilon = 0.1
        anonymizer = DiffPrivLaplaceMechanism.create_max_anonymizer(epsilon)
        self.assertIsInstance(anonymizer, DiffPrivMaxAnonymizer)
        self.assertEqual(anonymizer.epsilon, epsilon)

    def test_create_median_anonymizer(self):
        epsilon = 0.1
        anonymizer = DiffPrivLaplaceMechanism.create_median_anonymizer(epsilon)
        self.assertIsInstance(anonymizer, DiffPrivMedianAnonymizer)
        self.assertEqual(anonymizer.epsilon, epsilon)

    def test_create_proportion_anonymizer(self):
        epsilon = 0.1
        n = 10.0
        anonymizer = DiffPrivLaplaceMechanism.create_proportion_anonymizer(epsilon, n)
        self.assertIsInstance(anonymizer, DiffPrivProportionAnonymizer)
        self.assertEqual(anonymizer.epsilon, epsilon)
        self.assertEqual(anonymizer.n, n)

    def test_create_sum_anonymizer(self):
        epsilon = 1.0
        lower = 10.0
        upper = 99.0
        anonymizer = DiffPrivLaplaceMechanism.create_sum_anonymizer(
            epsilon, lower, upper
        )
        self.assertIsInstance(anonymizer, DiffPrivSumAnonymizer)
        self.assertEqual(anonymizer.epsilon, epsilon)
        self.assertEqual(anonymizer.lower, lower)
        self.assertEqual(anonymizer.upper, upper)

    def test_create_mean_anonymizer(self):
        epsilon = 1.0
        lower = 10.0
        upper = 99.0
        n = 100.0
        anonymizer = DiffPrivLaplaceMechanism.create_mean_anonymizer(
            epsilon, lower, upper, n
        )
        self.assertIsInstance(anonymizer, DiffPrivMeanAnonymizer)
        self.assertEqual(anonymizer.epsilon, epsilon)
        self.assertEqual(anonymizer.lower, lower)
        self.assertEqual(anonymizer.upper, upper)
        self.assertEqual(anonymizer.n, n)

    def test_create_variance_anonymizer(self):
        epsilon = 1.0
        lower = 10.0
        upper = 99.0
        n = 100.0
        anonymizer = DiffPrivLaplaceMechanism.create_variance_anonymizer(
            epsilon, lower, upper, n
        )
        self.assertIsInstance(anonymizer, DiffPrivVarianceAnonymizer)
        self.assertEqual(anonymizer.epsilon, epsilon)
        self.assertEqual(anonymizer.lower, lower)
        self.assertEqual(anonymizer.upper, upper)
        self.assertEqual(anonymizer.n, n)

    def test_anonymize_count_with_budget_single(self):
        expected_value = 87.58645513850368
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_count_with_budget(87.0, epsilon)
        np.testing.assert_almost_equal(anonymized, expected_value)

    def test_anonymize_count_with_budget_single_many(self):
        expected_values = np.array([87.5864551, 89.701297, 86.4519884])
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_count_with_budget(
            87.0, epsilon, size=3
        )
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_count_with_budget_multiple(self):
        expected_values = np.array([87.5864551, 437.701297])
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_count_with_budget(
            [87.0, 435.0], epsilon
        )
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_min_with_budget_single(self):
        expected_value = 87.58645513850368
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_min_with_budget(87.0, epsilon)
        np.testing.assert_almost_equal(anonymized, expected_value)

    def test_anonymize_min_with_budget_single_many(self):
        expected_values = np.array([87.5864551, 89.701297, 86.4519884])
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_min_with_budget(
            87.0, epsilon, size=3
        )
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_min_with_budget_multiple(self):
        expected_values = np.array([87.5864551, 437.701297])
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_min_with_budget(
            [87.0, 435.0], epsilon
        )
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_max_with_budget_single(self):
        expected_value = 87.58645513850368
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_max_with_budget(87.0, epsilon)
        np.testing.assert_almost_equal(anonymized, expected_value)

    def test_anonymize_max_with_budget_single_many(self):
        expected_values = np.array([87.5864551, 89.701297, 86.4519884])
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_max_with_budget(
            87.0, epsilon, size=3
        )
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_max_with_budget_multiple(self):
        expected_values = np.array([87.5864551, 437.701297])
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_max_with_budget(
            [87.0, 435.0], epsilon
        )
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_median_with_budget_single(self):
        expected_value = 87.58645513850368
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_median_with_budget(
            87.0, epsilon
        )
        np.testing.assert_almost_equal(anonymized, expected_value)

    def test_anonymize_median_with_budget_single_many(self):
        expected_values = np.array([87.5864551, 89.701297, 86.4519884])
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_median_with_budget(
            87.0, epsilon, size=3
        )
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_median_with_budget_multiple(self):
        expected_values = np.array([87.5864551, 437.701297])
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_median_with_budget(
            [87.0, 435.0], epsilon
        )
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_proportion_with_budget_single(self):
        expected_value = 87.05864551385037
        n = 10.0
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_proportion_with_budget(
            87.0, n, epsilon
        )
        np.testing.assert_almost_equal(anonymized, expected_value)

    def test_anonymize_proportion_with_budget_single_many(self):
        expected_values = np.array([87.0586455, 87.2701297, 86.9451988])
        n = 10.0
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_proportion_with_budget(
            87.0, n, epsilon, size=3
        )
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_proportion_with_budget_multiple(self):
        expected_values = np.array([87.0586455, 435.2701297])
        n = 10.0
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_proportion_with_budget(
            [87.0, 435.0], n, epsilon
        )
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_sum_with_budget_single(self):
        expected_value = 145.05905871186388
        lower = 10.0
        upper = 99.0
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_sum_with_budget(
            87.0, lower, upper, epsilon
        )
        np.testing.assert_almost_equal(anonymized, expected_value)

    def test_anonymize_sum_with_budget_single_many(self):
        expected_values = np.array([145.0590587, 354.4284063, 32.746848])
        lower = 10.0
        upper = 99.0
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_sum_with_budget(
            87.0, lower, upper, epsilon, size=3
        )
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_sum_with_budget_multiple(self):
        expected_values = np.array([145.0590587, 702.4284063])
        lower = 10.0
        upper = 99.0
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_sum_with_budget(
            [87.0, 435.0], lower, upper, epsilon
        )
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_mean_with_budget_single(self):
        expected_value = 87.52194507326827
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_mean_with_budget(
            87.0, lower, upper, n, epsilon
        )
        np.testing.assert_almost_equal(anonymized, expected_value)

    def test_anonymize_mean_with_budget_single_many(self):
        expected_values = np.array([87.5219451, 89.4041544, 86.5122696])
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_mean_with_budget(
            87.0, lower, upper, n, epsilon, size=3
        )
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_mean_with_budget_multiple(self):
        expected_values = np.array([87.5219451, 437.4041544])
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_mean_with_budget(
            [87.0, 435.0], lower, upper, n, epsilon
        )
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_variance_with_budget_single(self):
        expected_value = 133.45311152087612
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_variance_with_budget(
            87.0, lower, upper, n, epsilon
        )
        np.testing.assert_almost_equal(anonymized, expected_value)

    def test_anonymize_variance_with_budget_single_many(self):
        expected_values = np.array([133.4531115, 300.969738, 43.5919983])
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_variance_with_budget(
            87.0, lower, upper, n, epsilon, size=3
        )
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_variance_with_budget_multiple(self):
        expected_values = np.array([133.4531115, 648.969738])
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        self.set_seed()
        anonymized = DiffPrivLaplaceMechanism.anonymize_variance_with_budget(
            [87.0, 435.0], lower, upper, n, epsilon
        )
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_epsilon_getter(self):
        epsilon = 0.1
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.assertEqual(anonymizer.epsilon, epsilon)

    def test_anonymize_count_single(self):
        expected_value = 87.58645513850368
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_count(87.0)
        np.testing.assert_almost_equal(anonymized, expected_value)

    def test_anonymize_count_single_many(self):
        expected_values = np.array([87.5864551, 89.701297, 86.4519884])
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_count(87.0, size=3)
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_count_multiple(self):
        expected_values = np.array([87.5864551, 437.701297])
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_count([87.0, 435.0])
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_min_single(self):
        expected_value = 87.58645513850368
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_min(87.0)
        np.testing.assert_almost_equal(anonymized, expected_value)

    def test_anonymize_min_single_many(self):
        expected_values = np.array([87.5864551, 89.701297, 86.4519884])
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_min(87.0, size=3)
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_min_multiple(self):
        expected_values = np.array([87.5864551, 437.701297])
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_min([87.0, 435.0])
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_max_single(self):
        expected_value = 87.58645513850368
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_max(87.0)
        np.testing.assert_almost_equal(anonymized, expected_value)

    def test_anonymize_max_single_many(self):
        expected_values = np.array([87.5864551, 89.701297, 86.4519884])
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_max(87.0, size=3)
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_max_multiple(self):
        expected_values = np.array([87.5864551, 437.701297])
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_max([87.0, 435.0])
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_median_single(self):
        expected_value = 87.58645513850368
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_median(87.0)
        np.testing.assert_almost_equal(anonymized, expected_value)

    def test_anonymize_median_single_many(self):
        expected_values = np.array([87.5864551, 89.701297, 86.4519884])
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_median(87.0, size=3)
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_median_multiple(self):
        expected_values = np.array([87.5864551, 437.701297])
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_median([87.0, 435.0])
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_proportion_single(self):
        expected_value = 87.05864551385037
        n = 10.0
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_proportion(87.0, n)
        np.testing.assert_almost_equal(anonymized, expected_value)

    def test_anonymize_proportion_single_many(self):
        expected_values = np.array([87.0586455, 87.2701297, 86.9451988])
        n = 10.0
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_proportion(87.0, n, size=3)
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_proportion_multiple(self):
        expected_values = np.array([87.0586455, 435.2701297])
        n = 10.0
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_proportion([87.0, 435.0], n)
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_sum_single(self):
        expected_value = 145.05905871186388
        lower = 10.0
        upper = 99.0
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_sum(87.0, lower, upper)
        np.testing.assert_almost_equal(anonymized, expected_value)

    def test_anonymize_sum_single_many(self):
        expected_values = np.array([145.0590587, 354.4284063, 32.746848])
        lower = 10.0
        upper = 99.0
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_sum(87.0, lower, upper, size=3)
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_sum_multiple(self):
        expected_values = np.array([145.0590587, 702.4284063])
        lower = 10.0
        upper = 99.0
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_sum([87.0, 435.0], lower, upper)
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_mean_single(self):
        expected_value = 87.52194507326827
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_mean(87.0, lower, upper, n)
        np.testing.assert_almost_equal(anonymized, expected_value)

    def test_anonymize_mean_single_many(self):
        expected_values = np.array([87.5219451, 89.4041544, 86.5122696])
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_mean(87.0, lower, upper, n, size=3)
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_mean_multiple(self):
        expected_values = np.array([87.5219451, 437.4041544])
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_mean([87.0, 435.0], lower, upper, n)
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_variance_single(self):
        expected_value = 133.45311152087612
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_variance(87.0, lower, upper, n)
        np.testing.assert_almost_equal(anonymized, expected_value)

    def test_anonymize_variance_single_many(self):
        expected_values = np.array([133.4531115, 300.969738, 43.5919983])
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_variance(87.0, lower, upper, n, size=3)
        np.testing.assert_almost_equal(anonymized, expected_values)

    def test_anonymize_variance_multiple(self):
        expected_values = np.array([133.4531115, 648.969738])
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        anonymizer = DiffPrivLaplaceMechanism(epsilon)
        self.set_seed()
        anonymized = anonymizer.anonymize_variance([87.0, 435.0], lower, upper, n)
        np.testing.assert_almost_equal(anonymized, expected_values)



import re
from setuptools import setup, find_packages

with open("diffpriv_laplace/version.py", "r") as f:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE
    ).group(1)

if not version:
    raise RuntimeError("Cannot find version information")

url = "https://github.com/aleph-research/diff-priv-laplace-python"

setup(
    name="diff-priv-laplace-python",
    version=version,
    description="Python library for Laplace differential privacy",
    long_description="",
    keywords="laplace, differential, privacy",
    author="Elmar Langholz",
    author_email="langholz@gmail.com",
    url=url,
    download_url="{}/tarball/v{}".format(url, version),
    license="MIT",
    packages=find_packages(exclude="tests"),
    package_data={"README": ["README.md"]},
    install_requires=["numpy>=1.18.2"],
    zip_safe=False,
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



import unittest
import numpy as np
from diffpriv_laplace.anonymizer.median import DiffPrivMedianAnonymizer


class TestDiffPrivMedianAnonymizer(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def set_seed(self):
        np.random.seed(31337)

    def test_global_sensitivity_getter(self):
        epsilon = 1.0
        anonymizer = DiffPrivMedianAnonymizer(epsilon)
        self.assertEqual(anonymizer.global_sensitivity.value, 1.0)

    def test_epsilon_getter(self):
        epsilon = 1.0
        anonymizer = DiffPrivMedianAnonymizer(epsilon)
        self.assertEqual(anonymizer.epsilon, epsilon)

    def test_scale_getter(self):
        epsilon = 1.0
        scale = 1.0 / epsilon
        anonymizer = DiffPrivMedianAnonymizer(epsilon)
        self.assertEqual(anonymizer.scale, scale)

    def test_apply_single(self):
        expected_value = 87.58645513850368
        epsilon = 1.0
        anonymizer = DiffPrivMedianAnonymizer(epsilon)
        self.set_seed()
        value = anonymizer.apply(87.0)
        np.testing.assert_almost_equal(value, expected_value)

    def test_apply_single_many(self):
        expected_values = np.array([87.5864551, 89.701297, 86.4519884])
        epsilon = 1.0
        anonymizer = DiffPrivMedianAnonymizer(epsilon)
        self.set_seed()
        values = anonymizer.apply(87.0, size=3)
        np.testing.assert_almost_equal(values, expected_values)

    def test_apply_multiple(self):
        expected_values = np.array([87.5864551, 437.701297])
        epsilon = 1.0
        anonymizer = DiffPrivMedianAnonymizer(epsilon)
        self.set_seed()
        values = anonymizer.apply([87.0, 435.0])
        np.testing.assert_almost_equal(values, expected_values)



import unittest
import numpy as np
from itertools import combinations
from functools import reduce
from diffpriv_laplace import DiffPrivStatistics, DiffPrivStatisticKind


class TestDiffPrivStatistics(unittest.TestCase):
    epsilon = 100000
    decimal_places = 2

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def set_seed(self):
        np.random.seed(31337)

    def test_count_single(self):
        data = np.array([True, False, True, True] * 10)
        expected_value = 30.0
        self.set_seed()
        value = DiffPrivStatistics.count(data, self.epsilon)
        self.assertAlmostEqual(value, expected_value, self.decimal_places)

    def test_count_single_with_condition(self):
        data = np.array([0.1, -0.1, 0.1, 0.1] * 10)
        expected_value = 30.0

        def condition(data):
            return data >= 0.0

        self.set_seed()
        value = DiffPrivStatistics.count(data, self.epsilon, condition=condition)
        self.assertAlmostEqual(value, expected_value, self.decimal_places)

    def test_count_multiple(self):
        data = np.array([[True, False, True, True] * 10] * 3)
        expected_values = np.array([30.0] * 3)
        self.set_seed()
        value = DiffPrivStatistics.count(data, self.epsilon, axis=1)
        np.testing.assert_almost_equal(value, expected_values, self.decimal_places)

    def test_count_multiple_with_condition(self):
        data = np.array([[0.1, -0.1, 0.1, 0.1] * 10] * 3)
        expected_values = np.array([30.0] * 3)

        def condition(data):
            return data >= 0.0

        self.set_seed()
        value = DiffPrivStatistics.count(
            data, self.epsilon, condition=condition, axis=1
        )
        np.testing.assert_almost_equal(value, expected_values, self.decimal_places)

    def test_min_single(self):
        data = list(range(3, 20))
        data.reverse()
        data = np.array(data)
        expected_value = 3.0
        self.set_seed()
        value = DiffPrivStatistics.min(data, self.epsilon)
        self.assertAlmostEqual(value, expected_value, self.decimal_places)

    def test_min_multiple(self):
        data = list(range(3, 20))
        data.reverse()
        data = np.array([data] * 3)
        expected_values = np.array([3.0] * 3)
        self.set_seed()
        value = DiffPrivStatistics.min(data, self.epsilon, axis=1)
        np.testing.assert_almost_equal(value, expected_values, self.decimal_places)

    def test_max_single(self):
        data = np.array(list(range(3, 20)))
        expected_value = 19.0
        self.set_seed()
        value = DiffPrivStatistics.max(data, self.epsilon)
        self.assertAlmostEqual(value, expected_value, self.decimal_places)

    def test_max_multiple(self):
        data = np.array([list(range(3, 20))] * 3)
        expected_values = np.array([19.0] * 3)
        self.set_seed()
        value = DiffPrivStatistics.max(data, self.epsilon, axis=1)
        np.testing.assert_almost_equal(value, expected_values, self.decimal_places)

    def test_median_single(self):
        data = np.array(list(range(0, 20)) + [100.0])
        expected_value = 10.0
        self.set_seed()
        value = DiffPrivStatistics.median(data, self.epsilon)
        self.assertAlmostEqual(value, expected_value, self.decimal_places)

    def test_median_multiple(self):
        data = np.array([list(range(0, 20)) + [100.0]] * 3)
        expected_values = np.array([10.0] * 3)

        self.set_seed()
        value = DiffPrivStatistics.median(data, self.epsilon, axis=1)
        np.testing.assert_almost_equal(value, expected_values, self.decimal_places)

    def test_proportion_single(self):
        data = np.array([True, False, True, True] * 10)
        expected_value = 0.75
        self.set_seed()
        value = DiffPrivStatistics.proportion(data, self.epsilon)
        self.assertAlmostEqual(value, expected_value, self.decimal_places)

    def test_proportion_single_with_condition(self):
        data = np.array([0.1, -0.1, 0.1, 0.1] * 10)
        expected_value = 0.75

        def condition(data):
            return data >= 0.0

        self.set_seed()
        value = DiffPrivStatistics.proportion(data, self.epsilon, condition=condition)
        self.assertAlmostEqual(value, expected_value, self.decimal_places)

    def test_proportion_multiple(self):
        data = np.array([[True, False, True, True] * 10] * 3)
        expected_values = np.array([0.75] * 3)
        self.set_seed()
        value = DiffPrivStatistics.proportion(data, self.epsilon, axis=1)
        np.testing.assert_almost_equal(value, expected_values, self.decimal_places)

    def test_proportion_multiple_with_condition(self):
        data = np.array([[0.1, -0.1, 0.1, 0.1] * 10] * 3)
        expected_values = np.array([0.75] * 3)

        def condition(data):
            return data >= 0.0

        self.set_seed()
        value = DiffPrivStatistics.proportion(
            data, self.epsilon, condition=condition, axis=1
        )
        np.testing.assert_almost_equal(value, expected_values, self.decimal_places)

    def test_sum_single(self):
        data = np.array(list(range(0, 20)))
        expected_value = 190.0
        self.set_seed()
        value = DiffPrivStatistics.sum(data, self.epsilon)
        self.assertAlmostEqual(value, expected_value, self.decimal_places)

    def test_sum_multiple(self):
        data = np.array([list(range(0, 20))] * 3)
        expected_values = np.array([190.0] * 3)
        self.set_seed()
        value = DiffPrivStatistics.sum(data, self.epsilon, axis=1)
        np.testing.assert_almost_equal(value, expected_values, self.decimal_places)

    def test_mean_single(self):
        data = np.array(list(range(0, 20)) + [100.0])
        expected_value = 13.81
        self.set_seed()
        value = DiffPrivStatistics.mean(data, self.epsilon)
        self.assertAlmostEqual(value, expected_value, self.decimal_places)

    def test_mean_multiple(self):
        data = np.array([list(range(0, 20)) + [100.0]] * 3)
        expected_values = np.array([13.81] * 3)

        self.set_seed()
        value = DiffPrivStatistics.mean(data, self.epsilon, axis=1)
        np.testing.assert_almost_equal(value, expected_values, self.decimal_places)

    def test_variance_single(self):
        data = np.array(list(range(0, 20)) + [100.0])
        expected_value = 403.106
        self.set_seed()
        value = DiffPrivStatistics.variance(data, self.epsilon)
        self.assertAlmostEqual(value, expected_value, self.decimal_places)

    def test_variance_multiple(self):
        data = np.array([list(range(0, 20)) + [100.0]] * 3)
        expected_values = np.array([403.106] * 3)

        self.set_seed()
        value = DiffPrivStatistics.variance(data, self.epsilon, axis=1)
        np.testing.assert_almost_equal(value, expected_values, self.decimal_places)


class TestDiffPrivStatisticsKind(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_size(self):
        kinds = []
        for kind in DiffPrivStatisticKind:
            if kind != DiffPrivStatisticKind.all:
                kinds.append(kind)

        for r in range(1, len(kinds) + 1):
            combinations_values = combinations(kinds, r)
            for combination in list(combinations_values):
                values = reduce(lambda a, b: a | b, combination)
                self.assertEqual(values.size, r)

    def test_size_all(self):
        self.assertEqual(DiffPrivStatisticKind.all.size, len(DiffPrivStatisticKind) - 1)
        self.assertEqual(
            (DiffPrivStatisticKind.all | DiffPrivStatisticKind.all).size,
            len(DiffPrivStatisticKind) - 1,
        )



import unittest
import numpy as np
from diffpriv_laplace.anonymizer.mean import DiffPrivMeanAnonymizer


class TestDiffPrivMeanAnonymizer(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def set_seed(self):
        np.random.seed(31337)

    def test_length_getter(self):
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        anonymizer = DiffPrivMeanAnonymizer(epsilon, lower, upper, n)
        self.assertEqual(anonymizer.n, n)

    def test_lower_getter(self):
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        anonymizer = DiffPrivMeanAnonymizer(epsilon, lower, upper, n)
        self.assertEqual(anonymizer.lower, lower)

    def test_upper_getter(self):
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        anonymizer = DiffPrivMeanAnonymizer(epsilon, lower, upper, n)
        self.assertEqual(anonymizer.upper, upper)

    def test_global_sensitivity_getter(self):
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        gs_value = (upper - lower) / n
        anonymizer = DiffPrivMeanAnonymizer(epsilon, lower, upper, n)
        np.testing.assert_almost_equal(anonymizer.global_sensitivity.value, gs_value)

    def test_epsilon_getter(self):
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        anonymizer = DiffPrivMeanAnonymizer(epsilon, lower, upper, n)
        self.assertEqual(anonymizer.epsilon, epsilon)

    def test_scale_getter(self):
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        gs_value = (upper - lower) / n
        scale = gs_value / epsilon
        anonymizer = DiffPrivMeanAnonymizer(epsilon, lower, upper, n)
        self.assertEqual(anonymizer.scale, scale)

    def test_apply_single(self):
        expected_value = 87.52194507326827
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        anonymizer = DiffPrivMeanAnonymizer(epsilon, lower, upper, n)
        self.set_seed()
        value = anonymizer.apply(87.0)
        np.testing.assert_almost_equal(value, expected_value)

    def test_apply_single_many(self):
        expected_values = np.array([87.5219451, 89.4041544, 86.5122696])
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        anonymizer = DiffPrivMeanAnonymizer(epsilon, lower, upper, n)
        self.set_seed()
        values = anonymizer.apply(87.0, size=3)
        np.testing.assert_almost_equal(values, expected_values)

    def test_apply_multiple(self):
        expected_values = np.array([87.5219451, 437.4041544])
        lower = 10.0
        upper = 99.0
        n = 100.0
        epsilon = 1.0
        anonymizer = DiffPrivMeanAnonymizer(epsilon, lower, upper, n)
        self.set_seed()
        values = anonymizer.apply([87.0, 435.0])
        np.testing.assert_almost_equal(values, expected_values)



from diffpriv_laplace.global_sensitivity.sum import SumGlobalSensitivity
from diffpriv_laplace.anonymizer.base import DiffPrivAnonymizer


class DiffPrivSumAnonymizer(DiffPrivAnonymizer):
    def __init__(self, epsilon, lower, upper):
        gs = SumGlobalSensitivity(lower, upper)
        super().__init__(gs, epsilon)

    @property
    def lower(self):
        return self.global_sensitivity.lower

    @property
    def upper(self):
        return self.global_sensitivity.upper



from diffpriv_laplace.global_sensitivity.variance import VarianceGlobalSensitivity
from diffpriv_laplace.anonymizer.base import DiffPrivAnonymizer


class DiffPrivVarianceAnonymizer(DiffPrivAnonymizer):
    def __init__(self, epsilon, lower, upper, n):
        gs = VarianceGlobalSensitivity(lower, upper, n)
        super().__init__(gs, epsilon)

    @property
    def lower(self):
        return self.global_sensitivity.lower

    @property
    def upper(self):
        return self.global_sensitivity.upper

    @property
    def n(self):
        return self.global_sensitivity.n



from diffpriv_laplace.global_sensitivity.count import CountGlobalSensitivity
from diffpriv_laplace.anonymizer.base import DiffPrivAnonymizer


class DiffPrivCountAnonymizer(DiffPrivAnonymizer):
    def __init__(self, epsilon):
        gs = CountGlobalSensitivity()
        super().__init__(gs, epsilon)
